@@ -3,10 +3,14 @@
 
 package org.torproject.collector.bridgedescs;
 
+import org.torproject.collector.conf.Annotation;
 import org.torproject.collector.conf.Configuration;
 import org.torproject.collector.conf.ConfigurationException;
 import org.torproject.collector.conf.Key;
 import org.torproject.collector.cron.CollecTorMain;
+import org.torproject.descriptor.BridgeExtraInfoDescriptor;
+import org.torproject.descriptor.BridgeNetworkStatus;
+import org.torproject.descriptor.BridgeServerDescriptor;
 
 import org.apache.commons.codec.DecoderException;
 import org.apache.commons.codec.binary.Base64;
@@ -23,6 +27,7 @@ import java.io.FileWriter;
 import java.io.IOException;
 import java.io.StringReader;
 import java.io.UnsupportedEncodingException;
+import java.nio.file.Paths;
 import java.security.GeneralSecurityException;
 import java.security.SecureRandom;
 import java.text.ParseException;
@@ -53,9 +58,17 @@ public class SanitizedBridgesWriter extends CollecTorMain {
 
   private static final Logger logger = LoggerFactory.getLogger(
       SanitizedBridgesWriter.class);
+  private static final String BRIDGE_DESCRIPTORS = "bridge-descriptors";
 
+  /** Initialize configuration. */
   public SanitizedBridgesWriter(Configuration config) {
     super(config);
+    this.mapPathDescriptors.put("recent/bridge-descriptors/statuses",
+        BridgeNetworkStatus.class);
+    this.mapPathDescriptors.put("recent/bridge-descriptors/server-descriptors",
+        BridgeServerDescriptor.class);
+    this.mapPathDescriptors.put("recent/bridge-descriptors/extra-infos",
+        BridgeExtraInfoDescriptor.class);
   }
 
   private String rsyncCatString;
@@ -81,18 +94,30 @@ public class SanitizedBridgesWriter extends CollecTorMain {
 
   private SecureRandom secureRandom;
 
+  private String outputPathName;
+
+  private String recentPathName;
+
   @Override
   public String module() {
     return "bridgedescs";
   }
 
+  @Override
+  protected String syncMarker() {
+    return "Bridge";
+  }
+
   @Override
   protected void startProcessing() throws ConfigurationException {
 
+    outputPathName = Paths.get(config.getPath(Key.OutputPath).toString(),
+        BRIDGE_DESCRIPTORS).toString();
+    recentPathName = Paths.get(config.getPath(Key.RecentPath).toString(),
+        BRIDGE_DESCRIPTORS).toString();
     File bridgeDirectoriesDirectory =
-        config.getPath(Key.BridgeSnapshotsDirectory).toFile();
-    File sanitizedBridgesDirectory =
-        config.getPath(Key.SanitizedBridgesWriteDirectory).toFile();
+        config.getPath(Key.BridgeLocalOrigins).toFile();
+    File sanitizedBridgesDirectory = new File(outputPathName);
     File statsDirectory = config.getPath(Key.StatsPath).toFile();
 
     if (bridgeDirectoriesDirectory == null
@@ -138,7 +163,8 @@ public class SanitizedBridgesWriter extends CollecTorMain {
         while ((line = br.readLine()) != null) {
           String[] parts = line.split(",");
           if ((line.length() != ("yyyy-MM,".length() + 31 * 2)
-              && line.length() != ("yyyy-MM,".length() + 50 * 2))
+              && line.length() != ("yyyy-MM,".length() + 50 * 2)
+              && line.length() != ("yyyy-MM,".length() + 83 * 2))
               || parts.length != 2) {
             logger.warn("Invalid line in bridge-ip-secrets file "
                 + "starting with '" + line.substring(0, 7) + "'! "
@@ -218,8 +244,10 @@ public class SanitizedBridgesWriter extends CollecTorMain {
       scrubbedAddressPart = this.scrubIpv4Address(addressPart,
           fingerprintBytes, published);
     }
+    String scrubbedPort = this.scrubTcpPort(portPart, fingerprintBytes,
+        published);
     return (scrubbedAddressPart == null ? null :
-          scrubbedAddressPart + ":" + portPart);
+          scrubbedAddressPart + ":" + scrubbedPort);
   }
 
   private String scrubIpv4Address(String address, byte[] fingerprintBytes,
@@ -334,14 +362,42 @@ public class SanitizedBridgesWriter extends CollecTorMain {
     return sb.toString();
   }
 
+  private String scrubTcpPort(String portString, byte[] fingerprintBytes,
+      String published) throws IOException {
+    if (portString.equals("0")) {
+      return "0";
+    } else if (this.replaceIpAddressesWithHashes) {
+      if (this.persistenceProblemWithSecrets) {
+        /* There's a persistence problem, so we shouldn't scrub more TCP
+         * ports in this execution. */
+        return null;
+      }
+      byte[] hashInput = new byte[2 + 20 + 33];
+      int portNumber = Integer.parseInt(portString);
+      hashInput[0] = (byte) (portNumber >> 8);
+      hashInput[1] = (byte) portNumber;
+      System.arraycopy(fingerprintBytes, 0, hashInput, 2, 20);
+      String month = published.substring(0, "yyyy-MM".length());
+      byte[] secret = this.getSecretForMonth(month);
+      System.arraycopy(secret, 50, hashInput, 22, 33);
+      byte[] hashOutput = DigestUtils.sha256(hashInput);
+      int hashedPort = ((((hashOutput[0] & 0b1111_1111) << 8)
+          | (hashOutput[1] & 0b1111_1111)) >> 2) | 0b1100_0000_0000_0000;
+      return String.valueOf(hashedPort);
+    } else {
+      return "1";
+    }
+  }
+
   private byte[] getSecretForMonth(String month) throws IOException {
     if (!this.secretsForHashingIpAddresses.containsKey(month)
-        || this.secretsForHashingIpAddresses.get(month).length == 31) {
-      byte[] secret = new byte[50];
+        || this.secretsForHashingIpAddresses.get(month).length < 83) {
+      byte[] secret = new byte[83];
       this.secureRandom.nextBytes(secret);
       if (this.secretsForHashingIpAddresses.containsKey(month)) {
         System.arraycopy(this.secretsForHashingIpAddresses.get(month), 0,
-            secret, 0, 31);
+            secret, 0,
+            this.secretsForHashingIpAddresses.get(month).length);
       }
       if (month.compareTo(
           this.bridgeSanitizingCutOffTimestamp) < 0) {
@@ -362,8 +418,8 @@ public class SanitizedBridgesWriter extends CollecTorMain {
           bw.close();
         } catch (IOException e) {
           logger.warn("Could not store new secret "
-              + "to disk! Not calculating any IP address hashes in "
-              + "this execution!", e);
+              + "to disk! Not calculating any IP address or TCP port "
+              + "hashes in this execution!", e);
           this.persistenceProblemWithSecrets = true;
           throw new IOException(e);
         }
@@ -379,7 +435,8 @@ public class SanitizedBridgesWriter extends CollecTorMain {
    * Sanitizes a network status and writes it to disk.
    */
   public void sanitizeAndStoreNetworkStatus(byte[] data,
-      String publicationTime) throws ConfigurationException {
+      String publicationTime, String authorityFingerprint)
+      throws ConfigurationException {
 
     if (this.persistenceProblemWithSecrets) {
       /* There's a persistence problem, so we shouldn't scrub more IP
@@ -442,6 +499,16 @@ public class SanitizedBridgesWriter extends CollecTorMain {
 
           /* Parse the relevant parts of this r line. */
           String[] parts = line.split(" ");
+          if (parts.length < 9) {
+            logger.warn("Illegal line '" + line + "' in bridge network "
+                + "status.  Skipping descriptor.");
+            return;
+          }
+          if (!Base64.isBase64(parts[2])) {
+            logger.warn("Illegal base64 character in r line '" + parts[2]
+                + "'.  Skipping descriptor.");
+            return;
+          }
           fingerprintBytes = Base64.decodeBase64(parts[2] + "==");
           descPublicationTime = parts[4] + " " + parts[5];
           String address = parts[6];
@@ -470,11 +537,15 @@ public class SanitizedBridgesWriter extends CollecTorMain {
               fingerprintBytes,
               descPublicationTime);
           String nickname = parts[1];
+          String scrubbedOrPort = this.scrubTcpPort(orPort,
+              fingerprintBytes, descPublicationTime);
+          String scrubbedDirPort = this.scrubTcpPort(dirPort,
+              fingerprintBytes, descPublicationTime);
           scrubbed.append("r " + nickname + " "
               + hashedBridgeIdentityBase64 + " "
               + hashedDescriptorIdentifier + " " + descPublicationTime
-              + " " + scrubbedAddress + " " + orPort + " " + dirPort
-              + "\n");
+              + " " + scrubbedAddress + " " + scrubbedOrPort + " "
+              + scrubbedDirPort + "\n");
 
         /* Sanitize any addresses in a lines using the fingerprint and
          * descriptor publication time from the previous r line. */
@@ -495,7 +566,7 @@ public class SanitizedBridgesWriter extends CollecTorMain {
             || line.startsWith("p ") || line.equals("p")) {
           scrubbed.append(line + "\n");
 
-        /* There should be nothing else but r, w, p, and s lines in the
+        /* There should be nothing else but r, a, w, p, and s lines in the
          * network status.  If there is, we should probably learn before
          * writing anything to the sanitized descriptors. */
         } else {
@@ -546,16 +617,15 @@ public class SanitizedBridgesWriter extends CollecTorMain {
       File tarballFile = new File(
           this.sanitizedBridgesDirectory.getAbsolutePath() + "/" + syear
           + "/" + smonth + "/statuses/" + sday + "/" + syear + smonth
-          + sday + "-" + stime + "-"
-          + "4A0CCD2DDC7995083D73F5D667100C8A5831F16D");
-      File rsyncFile = new File(config.getPath(Key.RecentPath).toFile(),
-          "bridge-descriptors/statuses/" + tarballFile.getName());
+          + sday + "-" + stime + "-" + authorityFingerprint);
+      File rsyncFile = new File(recentPathName, "statuses/"
+          + tarballFile.getName());
       File[] outputFiles = new File[] { tarballFile, rsyncFile };
       for (File outputFile : outputFiles) {
         outputFile.getParentFile().mkdirs();
         BufferedWriter bw = new BufferedWriter(new FileWriter(
             outputFile));
-        bw.write("@type bridge-network-status 1.0\n");
+        bw.write(Annotation.Status.toString());
         bw.write("published " + publicationTime + "\n");
         bw.write(header.toString());
         for (String scrubbed : scrubbedLines.values()) {
@@ -595,6 +665,7 @@ public class SanitizedBridgesWriter extends CollecTorMain {
       String hashedBridgeIdentity = null;
       String address = null;
       String routerLine = null;
+      String scrubbedRouterLine = null;
       String scrubbedAddress = null;
       String masterKeyEd25519 = null;
       List<String> orAddresses = null;
@@ -611,7 +682,12 @@ public class SanitizedBridgesWriter extends CollecTorMain {
          * the bridge identity fingerprint for replacing the IP address in
          * the scrubbed version.  */
         } else if (line.startsWith("router ")) {
-          address = line.split(" ")[2];
+          String[] parts = line.split(" ");
+          if (parts.length != 6) {
+            logger.warn("Invalid router line: '" + line + "'.  Skipping.");
+            return;
+          }
+          address = parts[2];
           routerLine = line;
 
         /* Store or-address parts in a list and sanitize them when we have
@@ -671,6 +747,17 @@ public class SanitizedBridgesWriter extends CollecTorMain {
                 }
               }
             }
+            String[] routerLineParts = routerLine.split(" ");
+            String nickname = routerLineParts[1];
+            String scrubbedOrPort = this.scrubTcpPort(routerLineParts[3],
+                fingerprintBytes, published);
+            String scrubbedDirPort = this.scrubTcpPort(routerLineParts[4],
+                fingerprintBytes, published);
+            String scrubbedSocksPort = this.scrubTcpPort(
+                routerLineParts[5], fingerprintBytes, published);
+            scrubbedRouterLine = String.format("router %s %s %s %s %s%n",
+                nickname, scrubbedAddress, scrubbedOrPort,
+                scrubbedDirPort, scrubbedSocksPort);
           } catch (IOException e) {
             /* There's a persistence problem, so we shouldn't scrub more
              * IP addresses in this execution. */
@@ -692,14 +779,10 @@ public class SanitizedBridgesWriter extends CollecTorMain {
         /* When we reach the signature, we're done. Write the sanitized
          * descriptor to disk below. */
         } else if (line.startsWith("router-signature")) {
-          String[] routerLineParts = routerLine.split(" ");
-          scrubbedDesc = "router " + routerLineParts[1] + " "
-              + scrubbedAddress + " " + routerLineParts[3] + " "
-              + routerLineParts[4] + " " + routerLineParts[5] + "\n";
+          scrubbedDesc = scrubbedRouterLine;
           if (scrubbedOrAddresses != null) {
             for (String scrubbedOrAddress : scrubbedOrAddresses) {
-              scrubbedDesc = scrubbedDesc += "or-address "
-                  + scrubbedOrAddress + "\n";
+              scrubbedDesc += "or-address " + scrubbedOrAddress + "\n";
             }
           }
           scrubbedDesc += scrubbed.toString();
@@ -714,9 +797,19 @@ public class SanitizedBridgesWriter extends CollecTorMain {
             scrubbed.append("opt ");
             parts = line.substring(4).split(" ");
           }
+          if (parts.length > 3) {
+            logger.warn("extra-info-digest line contains more arguments than"
+                + "expected: '" + line + "'.  Skipping descriptor.");
+            return;
+          }
           scrubbed.append("extra-info-digest " + DigestUtils.shaHex(
               Hex.decodeHex(parts[1].toCharArray())).toUpperCase());
           if (parts.length > 2) {
+            if (!Base64.isBase64(parts[2])) {
+              logger.warn("Illegal base64 character in extra-info-digest line '"
+                  + line + "'.  Skipping descriptor.");
+              return;
+            }
             scrubbed.append(" " + Base64.encodeBase64String(
                 DigestUtils.sha256(Base64.decodeBase64(parts[2])))
                 .replaceAll("=", ""));
@@ -747,6 +840,11 @@ public class SanitizedBridgesWriter extends CollecTorMain {
           masterKeyEd25519FromIdentityEd25519 =
               this.parseMasterKeyEd25519FromIdentityEd25519(
               sb.toString());
+          if (masterKeyEd25519FromIdentityEd25519 == null) {
+            logger.warn("Could not parse master-key-ed25519 from "
+                + "identity-ed25519.  Skipping descriptor.");
+            return;
+          }
           String sha256MasterKeyEd25519 = Base64.encodeBase64String(
               DigestUtils.sha256(Base64.decodeBase64(
               masterKeyEd25519FromIdentityEd25519 + "=")))
@@ -777,6 +875,7 @@ public class SanitizedBridgesWriter extends CollecTorMain {
             || line.startsWith("platform ")
             || line.startsWith("opt protocols ")
             || line.startsWith("protocols ")
+            || line.startsWith("proto ")
             || line.startsWith("uptime ")
             || line.startsWith("bandwidth ")
             || line.startsWith("opt hibernating ")
@@ -915,7 +1014,7 @@ public class SanitizedBridgesWriter extends CollecTorMain {
         outputFile.getParentFile().mkdirs();
         BufferedWriter bw = new BufferedWriter(new FileWriter(
             outputFile, appendToFile));
-        bw.write("@type bridge-server-descriptor 1.1\n");
+        bw.write(Annotation.BridgeServer.toString());
         bw.write(scrubbedDesc);
         if (descriptorDigestSha256Base64 != null) {
           bw.write("router-digest-sha256 " + descriptorDigestSha256Base64
@@ -1011,6 +1110,11 @@ public class SanitizedBridgesWriter extends CollecTorMain {
          * its hash in the sanitized descriptor. */
         String[] parts = line.split(" ");
         if (line.startsWith("extra-info ")) {
+          if (parts.length < 3) {
+            logger.debug("Illegal line in extra-info descriptor: '" + line
+                + "'.  Skipping descriptor.");
+            return;
+          }
           hashedBridgeIdentity = DigestUtils.shaHex(Hex.decodeHex(
               parts[2].toCharArray())).toLowerCase();
           scrubbed = new StringBuilder("extra-info " + parts[1] + " "
@@ -1193,7 +1297,7 @@ public class SanitizedBridgesWriter extends CollecTorMain {
         outputFile.getParentFile().mkdirs();
         BufferedWriter bw = new BufferedWriter(new FileWriter(
             outputFile, appendToFile));
-        bw.write("@type bridge-extra-info 1.3\n");
+        bw.write(Annotation.BridgeExtraInfo.toString());
         bw.write(scrubbedDesc);
         if (descriptorDigestSha256Base64 != null) {
           bw.write("router-digest-sha256 " + descriptorDigestSha256Base64
@@ -1294,7 +1398,7 @@ public class SanitizedBridgesWriter extends CollecTorMain {
         - 3L * 24L * 60L * 60L * 1000L;
     Stack<File> allFiles = new Stack<File>();
     allFiles.add(new File(config.getPath(Key.RecentPath).toFile(),
-        "bridge-descriptors"));
+        BRIDGE_DESCRIPTORS));
     while (!allFiles.isEmpty()) {
       File file = allFiles.pop();
       if (file.isDirectory()) {
