@@ -9,11 +9,11 @@
 ####
 # Configuration section:
 # The following path should be adjusted, if the CollecTor server layout differs.
-# OUTDIR and TARBALLTARGETDIR have to be given absolute or relative to WORKDIR.
-ARCHIVEDIR="archive"
-WORKDIR="tarballs"
-OUTDIR="../out"
-TARBALLTARGETDIR="../data"
+# All paths should be given absolute.
+ARCHIVEDIR="/srv/collector.torproject.org/collector/archive"
+WORKDIR="/srv/collector.torproject.org/collector/tarballs"
+OUTDIR="/srv/collector.torproject.org/collector/out"
+TARBALLTARGETDIR="/srv/collector.torproject.org/collector/data"
 ### end of configuration section.
 #
 ### script start
@@ -24,9 +24,7 @@ YEARTWO=`date --date='7 days ago' +%Y`
 MONTHTWO=`date --date='7 days ago' +%m`
 CURRENTPATH=`pwd`
 
-if ! test -d $WORKDIR
-  then mkdir $WORKDIR
-fi
+mkdir -p $WORKDIR
 
 cd $WORKDIR
 
@@ -35,10 +33,7 @@ if ! test -d $OUTDIR
   exit 1
 fi
 
-if ! test -d $TARBALLTARGETDIR
-  then echo "$TARBALLTARGETDIR doesn't exist.  Exiting."
-  exit 1
-fi
+mkdir -p $TARBALLTARGETDIR
 
 TARBALLS=(
   exit-list-$YEARONE-$MONTHONE
