@@ -1,7 +1,12 @@
+/* Copyright 2016 The Tor Project
+ * See LICENSE for licensing information */
+
 package org.torproject.collector.conf;
 
 import java.net.URL;
 import java.nio.file.Path;
+import java.util.HashSet;
+import java.util.Set;
 
 /**
  * Enum containing all the properties keys of the configuration.
@@ -11,13 +16,23 @@ public enum Key {
 
   ShutdownGraceWaitMinutes(Long.class),
   RunOnce(Boolean.class),
-  ExitlistOutputDirectory(Path.class),
   ExitlistUrl(URL.class),
   InstanceBaseUrl(String.class),
   ArchivePath(Path.class),
   RecentPath(Path.class),
+  OutputPath(Path.class),
   IndexPath(Path.class),
   StatsPath(Path.class),
+  SyncPath(Path.class),
+  RelaySources(SourceType[].class),
+  BridgeSources(SourceType[].class),
+  ExitlistSources(SourceType[].class),
+  RelayCacheOrigins(String[].class),
+  RelayLocalOrigins(Path.class),
+  RelaySyncOrigins(URL[].class),
+  BridgeSyncOrigins(URL[].class),
+  BridgeLocalOrigins(Path.class),
+  ExitlistSyncOrigins(URL[].class),
   BridgedescsActivated(Boolean.class),
   BridgedescsOffsetMinutes(Integer.class),
   BridgedescsPeriodMinutes(Integer.class),
@@ -33,33 +48,19 @@ public enum Key {
   UpdateindexActivated(Boolean.class),
   UpdateindexOffsetMinutes(Integer.class),
   UpdateindexPeriodMinutes(Integer.class),
-  BridgeSnapshotsDirectory(Path.class),
-  CachedRelayDescriptorsDirectories(String[].class),
   CompressRelayDescriptorDownloads(Boolean.class),
-  DirectoryArchivesDirectory(Path.class),
-  DirectoryArchivesOutputDirectory(Path.class),
-  DownloadRelayDescriptors(Boolean.class),
   DirectoryAuthoritiesAddresses(String[].class),
   DirectoryAuthoritiesFingerprintsForVotes(String[].class),
-  DownloadCurrentConsensus(Boolean.class),
-  DownloadCurrentMicrodescConsensus(Boolean.class),
-  DownloadCurrentVotes(Boolean.class),
-  DownloadMissingServerDescriptors(Boolean.class),
-  DownloadMissingExtraInfoDescriptors(Boolean.class),
-  DownloadMissingMicrodescriptors(Boolean.class),
   DownloadAllServerDescriptors(Boolean.class),
   DownloadAllExtraInfoDescriptors(Boolean.class),
-  ImportCachedRelayDescriptors(Boolean.class),
-  ImportDirectoryArchives(Boolean.class),
   KeepDirectoryArchiveImportHistory(Boolean.class),
   ReplaceIpAddressesWithHashes(Boolean.class),
   BridgeDescriptorMappingsLimit(Integer.class),
-  SanitizedBridgesWriteDirectory(Path.class),
-  TorperfOutputDirectory(Path.class),
   TorperfFilesLines(String[].class),
-  TorperfSources(String[][].class);
+  TorperfHosts(String[][].class);
 
   private Class clazz;
+  private static Set<String> keys;
 
   /**
    * @param Class of key value.
@@ -72,4 +73,15 @@ public enum Key {
     return clazz;
   }
 
+  /** Verifies, if the given string corresponds to an enum value. */
+  public static boolean has(String someKey) {
+    if (null == keys) {
+      keys = new HashSet<>();
+      for (Key key : values()) {
+        keys.add(key.name());
+      }
+    }
+    return keys.contains(someKey);
+  }
+
 }
